/// <reference types="core-js" />
/** Class representing a semaphore
 * Semaphores are initialized with a number of permits that get aquired and released
 * over the lifecycle of the Semaphore. Functions can wait and stop executing until
 * a permit gets released (the method for which is called signal for historical reasons.)
 *
 * Locks that only allow one execution of a critical section are a special case of
 * Semaphores. To construct a lock, initialize a Semaphore with a permit count of 1.
 *
 * This Semaphore class is implemented with the help of promises that get returned
 * by functions that wait for permits to become available. This makes it possible
 * to use ES7 async/awaits to synchronize your code.
*/
export default class Semaphore {
    private permits;
    private promiseResolverQueue;
    /**
     * Creates a semaphore.
     * @param {number} permits The number of permits, i.e. things being allowed to run in parallel.
     * To create a lock that only lets one thing run at a time, set this to 1.
     * This number can also be negative.
     */
    constructor(permits: number);
    /**
     * Returns a promise used to wait for a permit to become available. This method is supposed
     * to be awaited on.
     * @return {Promise} A promise that gets resolved when execution is allowed to proceed.
     */
    wait(): Promise<boolean>;
    /**
     * Same as wait except the promise returned gets resolved with false if no permit becomes available in time.
     * @param {number} milliseconds The time spent waiting before the wait is aborted.
     * @return {Promise} A promise that gets resolved with true when execution is allowed to proceed or
     * false if the time given elapses before a permit becomes available.
     */
    waitFor(milliseconds: number): Promise<boolean>;
    /**
     * Synchronous function that tries to acquire a permit and returns true if successful, false otherwise.
     * @return {boolean} Whether a permit could be acquired.
     */
    tryAcquire(): boolean;
    /**
     * Acquires and returns all permits that are currently available
     * @return {number} Acquired permits.
     */
    drainPermits(): number;
    /**
     * Increases the number of permits by one. If there are other functions waiting, one of them will
     * continue to execute in a future iteration of the event loop.
     */
    signal(): void;
    /**
     * Schedules func to be called once a permit becomes available.
     * @param {function} func - The function to be executed.
     * @return {T} A promise that gets resolved with the return value of the function.
     */
    execute<T>(func: () => T): Promise<T>;
}
